@@ -6,20 +6,28 @@ import (
 	"encoding/json"
 	"fmt"
 	"os"
+	"os/exec"
 	"path/filepath"
-	"strconv"
 	"strings"
 
 	"github.com/abiosoft/colima/cli"
 	"github.com/abiosoft/colima/config"
 	"github.com/abiosoft/colima/config/configmanager"
-	"github.com/abiosoft/colima/util"
-	"github.com/sirupsen/logrus"
+	"github.com/abiosoft/colima/daemon/process/vmnet"
 )
 
-const (
-	LayerEnvVar = "COLIMA_LAYER_SSH_PORT"
-)
+// EnvLimaHome is the environment variable for the Lima directory.
+const EnvLimaHome = "LIMA_HOME"
+
+// Limactl is the limactl command.
+const Limactl = "limactl"
+
+func limactl(args ...string) *exec.Cmd {
+	cmd := cli.Command(Limactl, args...)
+	cmd.Env = append(cmd.Env, os.Environ()...)
+	cmd.Env = append(cmd.Env, EnvLimaHome+"="+LimaHome())
+	return cmd
+}
 
 // Instance returns current instance.
 func Instance() (InstanceInfo, error) {
@@ -41,8 +49,6 @@ func InstanceConfig() (config.Config, error) {
 //
 // TODO: unnecessary round-trip is done to get instance details from Lima.
 func IPAddress(profileID string) string {
-	// profile = toUserFriendlyName(profile)
-
 	const fallback = "127.0.0.1"
 	instance, err := getInstance(profileID)
 	if err != nil {
@@ -50,7 +56,11 @@ func IPAddress(profileID string) string {
 	}
 
 	if len(instance.Network) > 0 {
-		return getIPAddress(profileID, instance.Network[0].Interface)
+		for _, n := range instance.Network {
+			if n.Interface == vmnet.NetInterface {
+				return getIPAddress(profileID, n.Interface)
+			}
+		}
 	}
 
 	return fallback
@@ -83,11 +93,9 @@ func (i InstanceInfo) Config() (config.Config, error) {
 
 // ShowSSH runs the show-ssh command in Lima.
 // returns the ssh output, if in layer, and an error if any
-func ShowSSH(profileID string, layer bool) (resp struct {
-	Output    string
-	IPAddress string
-	Layer     bool
-	File      struct {
+func ShowSSH(profileID string) (resp struct {
+	Output string
+	File   struct {
 		Lima   string
 		Colima string
 	}
@@ -98,40 +106,18 @@ func ShowSSH(profileID string, layer bool) (resp struct {
 		return resp, fmt.Errorf("error retrieving ssh config: %w", err)
 	}
 
-	ip := IPAddress(profileID)
-	var port int
-	if layer {
-		port, _ = ubuntuSSHPort(profileID)
-		// if layer is active and public IP is available, use the fixed port
-		if port > 0 && ip != "127.0.0.1" {
-			port = 23
-		}
-	} else {
-		ip = "127.0.0.1"
-	}
-
-	resp.Output = replaceSSHConfig(sshConf, profileID, ip, port)
-	resp.IPAddress = ip
-	resp.Layer = port > 0
+	resp.Output = replaceSSHConfig(sshConf, profileID)
 	resp.File.Lima = ssh.File()
 	resp.File.Colima = config.SSHConfigFile()
 	return resp, nil
 }
 
-func replaceSSHConfig(conf string, profileID string, ip string, port int) string {
+func replaceSSHConfig(conf string, profileID string) string {
 	profileID = config.Profile(profileID).ID
-	name := config.Profile(profileID).ShortName
 
 	var out bytes.Buffer
 	scanner := bufio.NewScanner(strings.NewReader(conf))
 
-	hasPrefix := func(line, s string) (pad string, ok bool) {
-		if s != "" && strings.HasPrefix(strings.TrimSpace(line), s) {
-			return line[:strings.Index(line, s[:1])], true
-		}
-		return "", false
-	}
-
 	for scanner.Scan() {
 		line := scanner.Text()
 
@@ -139,21 +125,6 @@ func replaceSSHConfig(conf string, profileID string, ip string, port int) string
 			line = "Host " + profileID
 		}
 
-		if port > 0 {
-			if pad, ok := hasPrefix(line, "ControlPath "); ok {
-				configDir := filepath.Join(filepath.Dir(config.Dir()), name)
-				line = pad + "ControlPath " + strconv.Quote(filepath.Join(configDir, "ssh.sock"))
-			}
-
-			if pad, ok := hasPrefix(line, "Hostname "); ok {
-				line = pad + "Hostname " + ip
-			}
-
-			if pad, ok := hasPrefix(line, "Port"); ok {
-				line = pad + "Port " + strconv.Itoa(port)
-			}
-		}
-
 		_, _ = fmt.Fprintln(&out, line)
 	}
 	return out.String()
@@ -167,7 +138,7 @@ const (
 func getInstance(profileID string) (InstanceInfo, error) {
 	var i InstanceInfo
 	var buf bytes.Buffer
-	cmd := cli.Command("limactl", "list", profileID, "--json")
+	cmd := limactl("list", profileID, "--json")
 	cmd.Stderr = nil
 	cmd.Stdout = &buf
 
@@ -194,7 +165,7 @@ func Instances(ids ...string) ([]InstanceInfo, error) {
 	args := append([]string{"list", "--json"}, limaIDs...)
 
 	var buf bytes.Buffer
-	cmd := cli.Command("limactl", args...)
+	cmd := limactl(args...)
 	cmd.Stderr = nil
 	cmd.Stdout = &buf
 
@@ -217,8 +188,10 @@ func Instances(ids ...string) ([]InstanceInfo, error) {
 		}
 
 		if i.Running() {
-			if len(i.Network) > 0 && i.Network[0].Interface != "" {
-				i.IPAddress = getIPAddress(i.Name, i.Network[0].Interface)
+			for _, n := range i.Network {
+				if n.Interface == vmnet.NetInterface {
+					i.IPAddress = getIPAddress(i.Name, vmnet.NetInterface)
+				}
 			}
 			conf, _ := i.Config()
 			i.Runtime = getRuntime(conf)
@@ -238,8 +211,8 @@ func Instances(ids ...string) ([]InstanceInfo, error) {
 func getIPAddress(profileID, interfaceName string) string {
 	var buf bytes.Buffer
 	// TODO: this should be less hacky
-	cmd := cli.Command("limactl", "shell", profileID, "sh", "-c",
-		`ifconfig `+interfaceName+` | grep "inet addr:" | awk -F' ' '{print $2}' | awk -F':' '{print $2}'`)
+	cmd := limactl("shell", profileID, "sh", "-c",
+		`ip -4 addr show `+interfaceName+` | grep inet | awk -F' ' '{print $2 }' | cut -d/ -f1`)
 	cmd.Stderr = nil
 	cmd.Stdout = &buf
 
@@ -247,24 +220,6 @@ func getIPAddress(profileID, interfaceName string) string {
 	return strings.TrimSpace(buf.String())
 }
 
-func ubuntuSSHPort(profileID string) (int, error) {
-	var buf bytes.Buffer
-	cmd := cli.Command("limactl", "shell", profileID, "--", "sh", "-c", "echo $"+LayerEnvVar)
-	cmd.Stdout = &buf
-	cmd.Stderr = nil
-
-	if err := cmd.Run(); err != nil {
-		return 0, fmt.Errorf("cannot retrieve ubuntu layer SSH port: %w", err)
-	}
-
-	port, err := strconv.Atoi(strings.TrimSpace(buf.String()))
-	if err != nil {
-		return 0, fmt.Errorf("invalid ubuntu layer SSH port '%d': %w", port, err)
-	}
-
-	return port, nil
-}
-
 func getRuntime(conf config.Config) string {
 	var runtime string
 
@@ -281,45 +236,14 @@ func getRuntime(conf config.Config) string {
 	return runtime
 }
 
-var limaHome string
-
 // LimaHome returns the config directory for Lima.
 func LimaHome() string {
-	if limaHome != "" {
-		return limaHome
+	// if LIMA_HOME env var is set, obey it.
+	if dir := os.Getenv(EnvLimaHome); dir != "" {
+		return dir
 	}
 
-	home, err := func() (string, error) {
-		var buf bytes.Buffer
-		cmd := cli.Command("limactl", "info")
-		cmd.Stdout = &buf
-
-		if err := cmd.Run(); err != nil {
-			return "", fmt.Errorf("error retrieving lima info: %w", err)
-		}
-
-		var resp struct {
-			LimaHome string `json:"limaHome"`
-		}
-		if err := json.NewDecoder(&buf).Decode(&resp); err != nil {
-			return "", fmt.Errorf("error decoding json for lima info: %w", err)
-		}
-		if resp.LimaHome == "" {
-			return "", fmt.Errorf("error retrieving lima info, ensure lima version is >0.7.4")
-		}
-
-		return resp.LimaHome, nil
-	}()
-
-	if err != nil {
-		err = fmt.Errorf("error detecting Lima config directory: %w", err)
-		logrus.Warnln(err)
-		logrus.Warnln("falling back to default '$HOME/.lima'")
-		home = filepath.Join(util.HomeDir(), ".lima")
-	}
-
-	limaHome = home
-	return home
+	return config.LimaDir()
 }
 
 const colimaStateFileName = "colima.yaml"
